@@ -1,4 +1,3 @@
-/* $XFree86$ */
 /*
  * Copyright 2001-2003 Red Hat Inc., Durham, North Carolina.
  *
@@ -241,13 +240,15 @@ void dmxCommonKbdGetMap(DevicePtr pDev, KeySymsPtr pKeySyms, CARD8 *pModMap)
 
                                 /* Compute pModMap  */
     modifier_mapping     = XGetModifierMapping(priv->display);
-    for (i = 0; i < MAP_LENGTH; i++) pModMap[i] = 0;
+    for (i = 0; i < MAP_LENGTH; i++)
+        pModMap[i] = 0;
     for (j = 0; j < 8; j++) {
         int max_keypermod = modifier_mapping->max_keypermod;
         
         for (i = 0; i < max_keypermod; i++) {
             CARD8 keycode = modifier_mapping->modifiermap[j*max_keypermod + i];
-            if (keycode) pModMap[keycode] |= 1 << j;
+            if (keycode)
+                pModMap[keycode] |= 1 << j;
         }
     }
     XFreeModifiermap(modifier_mapping);
@@ -257,7 +258,6 @@ void dmxCommonKbdGetMap(DevicePtr pDev, KeySymsPtr pKeySyms, CARD8 *pModMap)
  * specified \a pDev. */
 void dmxCommonKbdGetInfo(DevicePtr pDev, DMXLocalInitInfoPtr info)
 {
-#ifdef XKB
     GETPRIVFROMPDEV;
     GETDMXINPUTFROMPRIV;
     char *pt;
@@ -283,7 +283,6 @@ void dmxCommonKbdGetInfo(DevicePtr pDev, DMXLocalInitInfoPtr info)
         if ((pt = strchr(info->names.keycodes, '+'))) *pt = '\0';
     }
     dmxCommonRestoreState(priv);
-#endif
 }
 
 /** Turn \a pDev on (i.e., take input from \a pDev). */
@@ -576,7 +575,6 @@ void dmxCommonSaveState(pointer private)
     if (dmxInput->console) priv = dmxInput->devs[0]->private;
     if (!priv->display || priv->stateSaved) return;
     DMXDBG0("dmxCommonSaveState\n");
-#ifdef XKB
     if (dmxUseXKB && (priv->xkb = XkbAllocKeyboard())) {
         if (XkbGetIndicatorMap(priv->display, XkbAllIndicatorsMask, priv->xkb)
             || XkbGetNames(priv->display, XkbAllNamesMask, priv->xkb)) {
@@ -595,7 +593,6 @@ void dmxCommonSaveState(pointer private)
             }
         }
     }
-#endif
 
     XGetKeyboardControl(priv->display, &ks);
     priv->savedKctrl.click              = ks.key_click_percent;
@@ -611,6 +608,7 @@ void dmxCommonSaveState(pointer private)
                         &priv->dmxLocal->kctrl);
 
     priv->savedModMap                   = XGetModifierMapping(priv->display);
+
     modmap                              = XNewModifiermap(0);
     XSetModifierMapping(priv->display, modmap);
     if (dmxInput->scrnIdx != -1)
@@ -627,25 +625,26 @@ void dmxCommonRestoreState(pointer private)
     int retcode = -1;
     CARD32 start;
 
-    if (dmxInput->console) priv = dmxInput->devs[0]->private;
-    if (!priv->stateSaved) return;
+    if (dmxInput->console)
+        priv = dmxInput->devs[0]->private;
+    if (!priv->stateSaved)
+        return;
     priv->stateSaved = 0;
     
     DMXDBG0("dmxCommonRestoreState\n");
-#ifdef XKB
     if (priv->xkb) {
         *priv->xkb->indicators = priv->savedIndicators;
         XkbSetIndicatorMap(priv->display, ~0, priv->xkb);
         XkbFreeKeyboard(priv->xkb, 0, True);
         priv->xkb = 0;
     }
-#endif
 
     for (start = GetTimeInMillis(); GetTimeInMillis() - start < 5000;) {
         CARD32 tmp;
         
         retcode = XSetModifierMapping(priv->display, priv->savedModMap);
-        if (retcode == MappingSuccess) break;
+        if (retcode == MappingSuccess)
+            break;
         if (retcode == MappingBusy)
             dmxLogInput(dmxInput, "Keyboard busy, waiting\n");
         else
