@@ -358,8 +358,7 @@ static unsigned char at2lnx[NUM_KEYCODES] =
 /** Create a private structure for use within this file. */
 pointer kbdLinuxCreatePrivate(DeviceIntPtr pKeyboard)
 {
-    myPrivate *priv = xalloc(sizeof(*priv));
-    memset(priv, 0, sizeof(*priv));
+    myPrivate *priv = calloc(1, sizeof(*priv));
     priv->fd        = -1;
     priv->pKeyboard = pKeyboard;
     return priv;
@@ -368,7 +367,7 @@ pointer kbdLinuxCreatePrivate(DeviceIntPtr pKeyboard)
 /** Destroy a private structure. */
 void kbdLinuxDestroyPrivate(pointer priv)
 {
-    if (priv) xfree(priv);
+    if (priv) free(priv);
 }
 
 /** Ring the bell.
@@ -698,7 +697,7 @@ static void kbdLinuxConvert(DevicePtr pDev,
 
             /* No auto-repeat? */
             if ((feed && !feed->ctrl.autoRepeat)
-                || priv->pKeyboard->key->modifierMap[keyCode]
+                || priv->pKeyboard->key->xkbInfo->desc->map->modmap[keyCode]
                 || (feed
                     && !(feed->ctrl.autoRepeats[keyCode >> 3]
                          & (1 << (keyCode & 7))))) return; /* Ignore */
